@@ -129,14 +129,18 @@ def connect_instance(dsrc_inst, verbose, args):
             # No password or we chose to prompt
             dsargs[SER_ROOT_PW] = getpass("Enter password for {} on {}: ".format(dsrc_inst['binddn'], dsrc_inst['uri']))
     elif not ds.can_autobind():
-        # No LDAPI, prompt for password
+        # No LDAPI, prompt for password, and bind DN if necessary
+        if dsrc_inst['binddn'] is None:
+            dn = ""
+            while dn == "":
+                dn = input("Enter Bind DN: ")
+            dsrc_inst['binddn'] = dn
         dsargs[SER_ROOT_PW] = getpass("Enter password for {} on {}: ".format(dsrc_inst['binddn'], dsrc_inst['uri']))
 
-    if 'binddn' in dsrc_inst:
-        # Allocate is an awful interface that we should stop using, but for now
-        # just directly map the dsrc_inst args in (remember, dsrc_inst DOES
-        # overlay cli args into the map ...)
-        dsargs[SER_ROOT_DN] = dsrc_inst['binddn']
+    # Allocate is an awful interface that we should stop using, but for now
+    # just directly map the dsrc_inst args in (remember, dsrc_inst DOES
+    # overlay cli args into the map ...)
+    dsargs[SER_ROOT_DN] = dsrc_inst['binddn']
 
     ds = DirSrv(verbose=verbose)
     ds.allocate(dsargs)
@@ -146,6 +150,8 @@ def connect_instance(dsrc_inst, verbose, args):
             usercert=dsrc_inst['tls_cert'],
             userkey=dsrc_inst['tls_key'],
             starttls=dsrc_inst['starttls'], connOnly=True)
+    if ds.serverid is not None and ds.serverid.startswith("slapd-"):
+        ds.serverid = ds.serverid.replace("slapd-", "", 1)
     return ds
 
 
@@ -164,7 +170,7 @@ def _generic_list(inst, basedn, log, manager_class, args=None):
     ol = mc.list()
     if len(ol) == 0:
         if args and args.json:
-            print(json.dumps({"type": "list", "items": []}))
+            print(json.dumps({"type": "list", "items": []}, indent=4))
         else:
             log.info("No objects to display")
     elif len(ol) > 0:
@@ -178,7 +184,7 @@ def _generic_list(inst, basedn, log, manager_class, args=None):
             else:
                 print(o_str)
         if args and args.json:
-            print(json.dumps(json_result))
+            print(json.dumps(json_result, indent=4))
 
 
 # Display these entries better!
@@ -210,7 +216,7 @@ def _generic_get_attr(inst, basedn, log, manager_class, args=None):
         else:
             print(mc.display_attr(attr).rstrip())
     if args.json:
-        print(json.dumps({"type": "entry", "dn": mc._dn, "attrs": vals}))
+        print(json.dumps({"type": "entry", "dn": mc._dn, "attrs": vals}, indent=4))
 
 
 def _generic_get_dn(inst, basedn, log, manager_class, dn, args=None):
