@@ -11,10 +11,12 @@ import logging
 import os
 import json
 import ldap
+import stat
+from shutil import copyfile
 from getpass import getpass
 from lib389._constants import ReplicaRole, DSRC_HOME
 from lib389.cli_base.dsrc import dsrc_to_repl_monitor
-from lib389.utils import is_a_dn
+from lib389.utils import is_a_dn, copy_with_permissions
 from lib389.replica import Replicas, ReplicationMonitor, BootstrapReplicationManager, Changelog5, ChangelogLDIF
 from lib389.tasks import CleanAllRUVTask, AbortCleanAllRUVTask
 from lib389._mapped_object import DSLdapObjects
@@ -111,7 +113,7 @@ def get_ruv(inst, basedn, log, args):
     ruv_dict = ruv.format_ruv()
     ruvs = ruv_dict['ruvs']
     if args and args.json:
-        log.info(json.dumps({"type": "list", "items": ruvs}))
+        log.info(json.dumps({"type": "list", "items": ruvs}, indent=4))
     else:
         add_gap = False
         for ruv in ruvs:
@@ -273,7 +275,7 @@ def list_suffixes(inst, basedn, log, args):
         suffixes.append(replica.get_suffix())
 
     if args.json:
-        log.info(json.dumps({"type": "list", "items": suffixes}))
+        log.info(json.dumps({"type": "list", "items": suffixes}, indent=4))
     else:
         if len(suffixes) == 0:
             log.info("There are no replicated suffixes")
@@ -287,7 +289,7 @@ def get_repl_status(inst, basedn, log, args):
     replica = replicas.get(args.suffix)
     status = replica.status(binddn=args.bind_dn, bindpw=args.bind_passwd)
     if args.json:
-        log.info(json.dumps({"type": "list", "items": status}))
+        log.info(json.dumps({"type": "list", "items": status}, indent=4))
     else:
         for agmt in status:
             log.info(agmt)
@@ -298,7 +300,7 @@ def get_repl_winsync_status(inst, basedn, log, args):
     replica = replicas.get(args.suffix)
     status = replica.status(binddn=args.bind_dn, bindpw=args.bind_passwd, winsync=True)
     if args.json:
-        log.info(json.dumps({"type": "list", "items": status}))
+        log.info(json.dumps({"type": "list", "items": status}, indent=4))
     else:
         for agmt in status:
             log.info(agmt)
@@ -374,15 +376,15 @@ def get_repl_monitor_info(inst, basedn, log, args):
 
         if connections:
             for connection_str in connections:
-                if len(connection_str.split(":")) != 4:
-                    raise ValueError(f"Connection string {connection_str} is in wrong format."
-                                     "It should be host:port:binddn:bindpw")
-                host_regex = connection_str.split(":")[0]
-                port_regex = connection_str.split(":")[1]
+                connection = connection_str.split(":")
+                if (len(connection) != 4 or not all([len(str) > 0 for str in connection])):
+                    raise ValueError(f"Please, fill in all Credential details. It should be host:port:binddn:bindpw")
+                host_regex = connection[0]
+                port_regex = connection[1]
                 if re.match(host_regex, host) and re.match(port_regex, port):
                     found = True
-                    binddn = connection_str.split(":")[2]
-                    bindpw = connection_str.split(":")[3]
+                    binddn = connection[2]
+                    bindpw = connection[3]
                     # Search for the password file or ask the user to write it
                     if bindpw.startswith("[") and bindpw.endswith("]"):
                         pwd_file_path = os.path.expanduser(bindpw[1:][:-1])
@@ -402,46 +404,63 @@ def get_repl_monitor_info(inst, basedn, log, args):
                 "bindpw": bindpw}
 
     repl_monitor = ReplicationMonitor(inst)
-    report_dict = repl_monitor.generate_report(get_credentials)
+    report_dict = repl_monitor.generate_report(get_credentials, args.json)
+    report_items = []
 
-    if args.json:
-        log.info(json.dumps({"type": "list", "items": report_dict}))
-    else:
-        for instance, report_data in report_dict.items():
-            found_alias = False
-            if args.aliases:
-                aliases = {al.split("=")[0]: al.split("=")[1] for al in args.aliases}
-            elif connection_data["aliases"]:
-                aliases = connection_data["aliases"]
-            else:
-                aliases = {}
-            if aliases:
-                for alias_name, alias_host_port in aliases.items():
-                    if alias_host_port.lower() == instance.lower():
-                        supplier_header = f"Supplier: {alias_name} ({instance})"
-                        found_alias = True
-                        break
-            if not found_alias:
-                supplier_header = f"Supplier: {instance}"
+    for instance, report_data in report_dict.items():
+        report_item = {}
+        found_alias = False
+        if args.aliases:
+            aliases = {al.split("=")[0]: al.split("=")[1] for al in args.aliases}
+        elif connection_data["aliases"]:
+            aliases = connection_data["aliases"]
+        else:
+            aliases = {}
+        if aliases:
+            for alias_name, alias_host_port in aliases.items():
+                if alias_host_port.lower() == instance.lower():
+                    supplier_header = f"{alias_name} ({instance})"
+                    found_alias = True
+                    break
+        if not found_alias:
+            supplier_header = f"{instance}"
+
+        if args.json:
+            report_item["name"] = supplier_header
+        else:
+            supplier_header = f"Supplier: {supplier_header}"
             log.info(supplier_header)
-            # Draw a line with the same length as the header
+
+        # Draw a line with the same length as the header
+        status = ""
+        if not args.json:
             log.info("-".join(["" for _ in range(0, len(supplier_header)+1)]))
-            if "status" in report_data and report_data["status"] == "Unavailable":
-                status = report_data["status"]
-                reason = report_data["reason"]
-                log.info(f"Status: {status}")
-                log.info(f"Reason: {reason}\n")
-            else:
-                for replica in report_data:
-                    replica_root = replica["replica_root"]
-                    replica_id = replica["replica_id"]
-                    maxcsn = replica["maxcsn"]
+        if report_data[0]["replica_status"].startswith("Unavailable"):
+            status = report_data[0]["replica_status"]
+            if not args.json:
+                log.info(f"Replica Status: {status}\n")
+        else:
+            for replica in report_data:
+                replica_root = replica["replica_root"]
+                replica_id = replica["replica_id"]
+                replica_status = replica["replica_status"]
+                maxcsn = replica["maxcsn"]
+                if not args.json:
                     log.info(f"Replica Root: {replica_root}")
                     log.info(f"Replica ID: {replica_id}")
+                    log.info(f"Replica Status: {replica_status}")
                     log.info(f"Max CSN: {maxcsn}\n")
-                    for agreement_status in replica["agmts_status"]:
+                for agreement_status in replica["agmts_status"]:
+                    if not args.json:
                         log.info(agreement_status)
 
+        if args.json:
+            report_item["data"] = report_data
+            report_items.append(report_item)
+
+    if args.json:
+        log.info(json.dumps({"type": "list", "items": report_items}, indent=4))
+
 
 def create_cl(inst, basedn, log, args):
     cl = Changelog5(inst)
@@ -593,7 +612,7 @@ def list_agmts(inst, basedn, log, args):
         else:
             log.info(agmt.display())
     if args.json:
-        log.info(json.dumps(result))
+        log.info(json.dumps(result, indent=4))
 
 
 def add_agmt(inst, basedn, log, args):
@@ -644,7 +663,7 @@ def add_agmt(inst, basedn, log, args):
         properties['nsds5replicastripattrs'] = args.strip_list
 
     # We do need the bind dn and credentials for none-sasl bind methods
-    if (bind_method == 'simple' or 'sslclientauth') and (args.bind_dn is None or args.bind_passwd is None):
+    if (bind_method in ('simple', 'sslclientauth')) and (args.bind_dn is None or args.bind_passwd is None):
         raise ValueError("You need to set the bind dn (--bind-dn) and the password (--bind-passwd) for bind method ({})".format(bind_method))
 
     # Create the agmt
@@ -693,7 +712,7 @@ def check_init_agmt(inst, basedn, log, args):
     elif error:
         status = "Agreement initialization failed: " + error
     if args.json:
-        log.info(json.dumps(status))
+        log.info(json.dumps(status, indent=4))
     else:
         log.info(status)
 
@@ -768,7 +787,7 @@ def list_winsync_agmts(inst, basedn, log, args):
         else:
             log.info(agmt.display())
     if args.json:
-        log.info(json.dumps(result))
+        log.info(json.dumps(result, indent=4))
 
 
 def add_winsync_agmt(inst, basedn, log, args):
@@ -888,7 +907,7 @@ def check_winsync_init_agmt(inst, basedn, log, args):
     elif error:
         status = "Agreement initialization failed."
     if args.json:
-        log.info(json.dumps(status))
+        log.info(json.dumps(status, indent=4))
     else:
         log.info(status)
 
@@ -926,7 +945,7 @@ def run_cleanallruv(inst, basedn, log, args):
     clean_task.create(properties=properties)
     rdn = clean_task.rdn
     if args.json:
-        log.info(json.dumps(rdn))
+        log.info(json.dumps(rdn, indent=4))
     else:
         log.info('Created task ' + rdn)
 
@@ -951,7 +970,7 @@ def list_cleanallruv(inst, basedn, log, args):
         else:
             log.info(task.display())
     if args.json:
-        log.info(json.dumps(result))
+        log.info(json.dumps(result, indent=4))
     else:
         if not tasks_found:
             log.info("No CleanAllRUV tasks found")
@@ -986,7 +1005,7 @@ def list_abort_cleanallruv(inst, basedn, log, args):
         else:
             log.info(task.display())
     if args.json:
-        log.info(json.dumps(result))
+        log.info(json.dumps(result, indent=4))
     else:
         if not tasks_found:
             log.info("No CleanAllRUV abort tasks found")
@@ -1000,7 +1019,8 @@ def dump_cl(inst, basedn, log, args):
     if not args.changelog_ldif:
         replicas.process_and_dump_changelog(replica_roots=args.replica_roots,
                                             csn_only=args.csn_only,
-                                            preserve_ldif_done=args.preserve_ldif_done)
+                                            preserve_ldif_done=args.preserve_ldif_done,
+                                            log=log)
     else:
         try:
             assert os.path.exists(args.changelog_ldif)
@@ -1013,6 +1033,39 @@ def dump_cl(inst, basedn, log, args):
             cl_ldif.decode()
 
 
+def restore_cl_ldif(inst, basedn, log, args):
+    user_ldif = os.path.abspath(args.LDIF_PATH[0])
+    try:
+        assert os.path.exists(user_ldif)
+    except AssertionError:
+        raise FileNotFoundError(f"File {args.LDIF_PATH[0]} was not found")
+
+    replicas = Replicas(inst)
+    replica = replicas.get(args.replica_root[0])
+    replica_name = replica.get_attr_val_utf8_l("nsDS5ReplicaName")
+    target_ldif = f'{replica_name}.ldif'
+    target_ldif_exists = os.path.exists(target_ldif)
+    cl = Changelog5(inst)
+    cl_dir = cl.get_attr_val_utf8_l("nsslapd-changelogdir")
+    cl_dir_file = [i.lower() for i in os.listdir(cl_dir) if i.lower().startswith(replica_name)][0]
+    cl_dir_stat = os.stat(os.path.join(cl_dir, cl_dir_file))
+    # Make sure we don't remove existing files
+    if target_ldif_exists:
+        copy_with_permissions(target_ldif, f'{target_ldif}.backup')
+    copyfile(user_ldif, target_ldif)
+    os.chown(target_ldif, cl_dir_stat[stat.ST_UID], cl_dir_stat[stat.ST_GID])
+    os.chmod(target_ldif, cl_dir_stat[stat.ST_MODE])
+    replicas.restore_changelog(replica_roots=args.replica_root, log=log)
+    os.remove(target_ldif)
+    if target_ldif_exists:
+        os.rename(f'{target_ldif}.backup', target_ldif)
+
+
+def restore_cl_dir(inst, basedn, log, args):
+    replicas = Replicas(inst)
+    replicas.restore_changelog(replica_roots=args.REPLICA_ROOTS, log=log)
+
+
 def create_parser(subparsers):
 
     ############################################
@@ -1103,26 +1156,39 @@ def create_parser(subparsers):
     repl_get_cl = repl_subcommands.add_parser('get-changelog', help='Display replication changelog attributes.')
     repl_get_cl.set_defaults(func=get_cl)
 
-    repl_set_cl = repl_subcommands.add_parser('dump-changelog', help='Decode Directory Server replication change log and dump it to an LDIF')
-    repl_set_cl.set_defaults(func=dump_cl)
-    repl_set_cl.add_argument('-c', '--csn-only', action='store_true',
-                             help="Dump and interpret CSN only. This option can be used with or without -i option.")
-    repl_set_cl.add_argument('-l', '--preserve-ldif-done', action='store_true',
-                             help="Preserve generated ldif.done files from changelogdir.")
-    repl_set_cl.add_argument('-i', '--changelog-ldif',
-                             help="If you already have a ldif-like changelog, but the changes in that file are encoded,"
-                                  " you may use this option to decode that ldif-like changelog. It should be base64 encoded.")
-    repl_set_cl.add_argument('-o', '--output-file', help="Path name for the final result. Default to STDOUT if omitted.")
-    repl_set_cl.add_argument('-r', '--replica-roots', nargs="+",
-                             help="Specify replica roots whose changelog you want to dump. The replica "
-                                  "roots may be seperated by comma. All the replica roots would be dumped if the option is omitted.")
+    repl_dump_cl = repl_subcommands.add_parser('dump-changelog', help='Decode Directory Server replication change log and dump it to an LDIF')
+    repl_dump_cl.set_defaults(func=dump_cl)
+    repl_dump_cl.add_argument('-c', '--csn-only', action='store_true',
+                              help="Dump and interpret CSN only. This option can be used with or without -i option.")
+    repl_dump_cl.add_argument('-l', '--preserve-ldif-done', action='store_true',
+                              help="Preserve generated ldif.done files from changelogdir.")
+    repl_dump_cl.add_argument('-i', '--changelog-ldif',
+                              help="If you already have a ldif-like changelog, but the changes in that file are encoded,"
+                                   " you may use this option to decode that ldif-like changelog. It should be base64 encoded.")
+    repl_dump_cl.add_argument('-o', '--output-file', help="Path name for the final result. Default to STDOUT if omitted.")
+    repl_dump_cl.add_argument('-r', '--replica-roots', nargs="+",
+                              help="Specify replica roots whose changelog you want to dump. The replica "
+                                   "roots may be seperated by comma. All the replica roots would be dumped if the option is omitted.")
+
+    repl_restore_cl = repl_subcommands.add_parser('restore-changelog',
+                                                  help='Restore Directory Server replication change log from LDIF file or change log directory')
+    restore_subcommands = repl_restore_cl.add_subparsers(help='Restore Replication Changelog')
+    restore_ldif = restore_subcommands.add_parser('from-ldif', help='Restore a single LDIF file.')
+    restore_ldif.set_defaults(func=restore_cl_ldif)
+    restore_ldif.add_argument('LDIF_PATH', nargs=1, help='The path of changelog LDIF file.')
+    restore_ldif.add_argument('-r', '--replica-root', nargs=1, required=True,
+                              help="Specify one replica root whose changelog you want to restore. "
+                                   "The replica root will be consumed from the LDIF file name if the option is omitted.")
+
+    restore_changelogdir = restore_subcommands.add_parser('from-changelogdir', help='Restore LDIF files from changelogdir.')
+    restore_changelogdir.set_defaults(func=restore_cl_dir)
+    restore_changelogdir.add_argument('REPLICA_ROOTS', nargs="+",
+                                      help="Specify replica roots whose changelog you want to restore. The replica "
+                                           "roots may be seperated by comma. All the replica roots would be dumped if the option is omitted.")
 
     repl_set_parser = repl_subcommands.add_parser('set', help='Set an attribute in the replication configuration')
     repl_set_parser.set_defaults(func=set_repl_config)
     repl_set_parser.add_argument('--suffix', required=True, help='The DN of the replication suffix')
-    repl_set_parser.add_argument('--replica-id', help="The Replication Identifier number")
-    repl_set_parser.add_argument('--replica-role', help="The Replication role: master, hub, or consumer")
-
     repl_set_parser.add_argument('--repl-add-bind-dn', help="Add a bind (supplier) DN")
     repl_set_parser.add_argument('--repl-del-bind-dn', help="Remove a bind (supplier) DN")
     repl_set_parser.add_argument('--repl-add-ref', help="Add a replication referral (for consumers only)")
