@@ -213,6 +213,19 @@ class Config(DSLdapObject):
             report['fix'] = report['fix'].replace('YOUR_INSTANCE', self._instance.serverid)
             yield report
 
+    def disable_plaintext_port(self):
+        """
+        Configure the server to not-provide the plaintext port.
+        """
+        self.set('nsslapd-port', '0')
+
+    def enable_plaintext_port(self, port):
+        """
+        Configure the server to provide the plaintext port on the specified port number.
+        """
+        self.set('nsslapd-port', port)
+
+
 class Encryption(DSLdapObject):
     """
         Manage "cn=encryption,cn=config" tree, including:
@@ -238,7 +251,7 @@ class Encryption(DSLdapObject):
 
     def _lint_check_tls_version(self):
         tls_min = self.get_attr_val('sslVersionMin')
-        if tls_min < ensure_bytes('TLS1.1'):
+        if tls_min is not None and tls_min < ensure_bytes('TLS1.1'):
             report = copy.deepcopy(DSELE0001)
             report['fix'] = report['fix'].replace('YOUR_INSTANCE', self._instance.serverid)
             yield report
